@@ -600,7 +600,6 @@ class DataBuffer(object):
             name = '%s/%s-%s-%s.gwf' % (pattern, self.beg, s, self.dur)
             # check that file actually exists, else abort now
             if not os.path.exists(name):
-                logging.info("%s not found yet", os.path.basename(name))
                 raise RuntimeError
 
             keys.append(name)
@@ -643,7 +642,7 @@ class DataBuffer(object):
                 return None
             else:
                 # I am too early to give up on this frame, so we should try again
-                time.sleep(1)
+                time.sleep(0.1)
                 return self.attempt_advance(blocksize, timeout=timeout)
 
 class StatusBuffer(DataBuffer):
