@@ -928,7 +928,6 @@ class GaussianNoise(BaseGaussianNoise):
             # now try returning again
             return getattr(self._current_stats, '{}_optimal_snrsq'.format(det))
 
-
 #
 # =============================================================================
 #
@@ -936,6 +935,8 @@ class GaussianNoise(BaseGaussianNoise):
 #
 # =============================================================================
 #
+
+
 def get_values_from_injection(cp, injection_file, update_cp=True):
     """Replaces all FROM_INJECTION values in a config file with the
     corresponding value from the injection.
@@ -1046,9 +1047,11 @@ def get_values_from_injection(cp, injection_file, update_cp=True):
     return replace_params
 
 
-def create_waveform_generator(variable_params, data, waveform_transforms=None,
-                              recalibration=None, gates=None,
-                              **static_params):
+def create_waveform_generator(
+                variable_params, data, waveform_transforms=None,
+                recalibration=None, gates=None,
+                generator_class=generator.FDomainDetFrameGenerator,
+                **static_params):
     """Creates a waveform generator for use with a model.
 
     Parameters
@@ -1089,6 +1092,7 @@ def create_waveform_generator(variable_params, data, waveform_transforms=None,
         approximant = static_params['approximant']
     except KeyError:
         raise ValueError("no approximant provided in the static args")
+
     generator_function = generator.select_waveform_generator(approximant)
     # get data parameters; we'll just use one of the data to get the
     # values, then check that all the others are the same
@@ -1103,7 +1107,7 @@ def create_waveform_generator(variable_params, data, waveform_transforms=None,
                         d.start_time == start_time]):
                 raise ValueError("data must all have the same delta_t, "
                                  "delta_f, and start_time")
-    waveform_generator = generator.FDomainDetFrameGenerator(
+    waveform_generator = generator_class(
         generator_function, epoch=start_time,
         variable_args=variable_params, detectors=list(data.keys()),
         delta_f=delta_f, delta_t=delta_t,
