@@ -1015,14 +1015,15 @@ class CartesianSpinToChiP(BaseTransform):
 class LambdaFromTOVFile(BaseTransform):
     """Transforms mass values corresponding to Lambda values for a given EOS
     interpolating from the mass-Lambda data for that EOS read in from an
-    external ASCII file. The interpolation of the mass-Lambda data is a
-    one-dimensional piecewise linear interpolation. The mass values to be
-    transformed are assumed to be detector frame masses, so a distance should
-    be provided along with the mass for transformation to the source frame
-    mass before the Lambda values are extracted from the interpolation. If
-    the mass value inputted is in the source frame, then provide distance=0.
-    If the transform is read in from a config file, an example code block
-    would be:
+    external ASCII file.
+
+    The interpolation of the mass-Lambda data is a one-dimensional piecewise
+    linear interpolation. If the ``redshift_mass`` keyword argument is ``True``
+    (the default), the mass values to be transformed are assumed to be detector
+    frame masses. In that case, a distance should be provided along with the
+    mass for transformation to the source frame mass before the Lambda values
+    are extracted from the interpolation. If the transform is read in from a
+    config file, an example code block would be:
 
     .. code-block:: ini
 
@@ -1045,6 +1046,20 @@ class LambdaFromTOVFile(BaseTransform):
         lambda_param = lambda1
         mass_lambda_file = filepath
 
+    If your prior is in terms of the source-frame masses (``srcmass``), then
+    you can shut off the redshifting by adding ``do-not-redshift-mass`` to the
+    config file. In this case you do not need to worry about a distance.
+    Example:
+
+    .. code-block:: ini
+
+        [{section}-lambda1]
+        name = lambda_from_tov_file
+        mass_param = srcmass1
+        lambda_param = lambda1
+        mass_lambda_file = filepath
+        do-not-redshift-mass =
+
     Parameters
     ----------
     mass_param : str
@@ -1056,8 +1071,12 @@ class LambdaFromTOVFile(BaseTransform):
         Path of the mass-Lambda data file. The first column in the data file
         should contain mass values, and the second column Lambda values.
     distance : float, optional
-        The distance (in Mpc) of the source. Used to redshift the mass. If
+        The distance (in Mpc) of the source. Used to redshift the mass. Needed
+        if ``redshift_mass`` is True and no distance parameter exists If
         None, then a distance must be provided to the transform.
+    redshift_mass : bool, optional
+        Redshift the mass parameters when computing the lambdas. Default is
+        False.
     file_columns : list of str, optional
         The names and order of columns in the ``mass_lambda_file``. Must
         contain at least 'mass' and 'lambda'. If not provided, will assume the
@@ -1066,10 +1085,11 @@ class LambdaFromTOVFile(BaseTransform):
     name = 'lambda_from_tov_file'
 
     def __init__(self, mass_param, lambda_param, mass_lambda_file,
-                 distance=None, file_columns=None):
+                 distance=None, redshift_mass=True, file_columns=None):
         self._mass_lambda_file = mass_lambda_file
         self._mass_param = mass_param
         self._lambda_param = lambda_param
+        self.redshift_mass = redshift_mass
         self._distance = distance
         self._inputs = [mass_param, 'distance']
         self._outputs = [lambda_param]
@@ -1116,7 +1136,7 @@ class LambdaFromTOVFile(BaseTransform):
         return self._distance
 
     @staticmethod
-    def lambda_from_tov_data(m, d, mass_data, lambda_data):
+    def lambda_from_tov_data(m_src, mass_data, lambda_data):
         """Returns Lambda corresponding to a given mass interpolating from the
         TOV data.
 
@@ -1124,8 +1144,6 @@ class LambdaFromTOVFile(BaseTransform):
         ----------
         m : float
             Value of the mass.
-        d : float
-            Value of distance.
         mass_data : array
             Mass array from the Lambda-M curve of an EOS.
         lambda_data : array
@@ -1136,8 +1154,11 @@ class LambdaFromTOVFile(BaseTransform):
         lambdav : float
             The Lambda corresponding to the mass `m` for the EOS considered.
         """
-        m_src = m/(1.0 + cosmology.redshift(abs(d)))
-        lambdav = numpy.interp(m_src, mass_data, lambda_data)
+        if m_src > mass_data.max():
+            # assume black hole
+            lambdav = 0.
+        else:
+            lambdav = numpy.interp(m_src, mass_data, lambda_data)
         return lambdav
 
     def transform(self, maps):
@@ -1156,21 +1177,38 @@ class LambdaFromTOVFile(BaseTransform):
             with the original variable name and value(s).
         """
         m = maps[self._mass_param]
-        if self._distance is not None:
-            d = self._distance
+        if self.redshift_mass:
+            if self._distance is not None:
+                d = self._distance
+            else:
+                try:
+                    d = maps['distance']
+                except KeyError as e:
+                    logging.warning("Either provide distance samples in the "
+                                    "list of samples to be transformed, or "
+                                    "provide a fixed distance value as input "
+                                    "when initializing LambdaFromTOVFile.")
+                    raise e
+            shift = 1./(1.0 + cosmology.redshift(abs(d)))
         else:
-            try:
-                d = maps['distance']
-            except KeyError as e:
-                logging.warning("Either provide `distance` samples in the "
-                                "list of samples to be transformed, or "
-                                "provide a fixed `distance` value as input "
-                                "when initializing `LambdaFromTOVFile`.")
-                raise e
+            shift = 1.
         out = {self._lambda_param : self.lambda_from_tov_data(
-            m, d, self._data['mass'], self._data['lambda'])}
+            m*shift, self._data['mass'], self._data['lambda'])}
         return self.format_output(maps, out)
 
+    @classmethod
+    def from_config(cls, cp, section, outputs):
+        # see if we're redshifting masses
+        if cp.has_option('-'.join([section, outputs]), 'do-not-redshift-mass'):
+            additional_opts = {'redshift_mass': False}
+            skip_opts = ['do-not-redshift-mass']
+        else:
+            additional_opts = None
+            skip_opts = None
+        return super(LambdaFromTOVFile, cls).from_config(
+            cp, section, outputs, skip_opts=skip_opts,
+            additional_opts=additional_opts)
+
 
 class LambdaFromMultipleTOVFiles(BaseTransform):
     """Uses multiple equation of states.
@@ -1195,20 +1233,21 @@ class LambdaFromMultipleTOVFiles(BaseTransform):
     """
 
     name = 'lambda_from_multiple_tov_files'
-    
+
     def __init__(self, mass_param, lambda_param, map_file, distance=None,
-                 file_columns=None):
+                 redshift_mass=True, file_columns=None):
         self._map_file = map_file
         self._mass_param = mass_param
         self._lambda_param = lambda_param
         self._distance = distance
+        self.redshift_mass = redshift_mass
         self._inputs = [mass_param, 'eos', 'distance']
         self._outputs = [lambda_param]
         # create a dictionary of the EOS files from the map_file
-        self._eos_files = {} 
+        self._eos_files = {}
         with open(self._map_file, 'r') as fp:
             for line in fp:
-                fname = line.rstrip('\n') 
+                fname = line.rstrip('\n')
                 eosidx = int(os.path.basename(fname).split('.')[0])
                 self._eos_files[eosidx] = os.path.abspath(fname)
         # create an eos cache for fast load later
@@ -1217,8 +1256,8 @@ class LambdaFromMultipleTOVFiles(BaseTransform):
             file_columns = ('radius', 'mass', 'lambda')
         self._file_columns = file_columns
         super(LambdaFromMultipleTOVFiles, self).__init__()
-    
-    
+
+
     @property
     def mass_param(self):
         """Returns the input mass parameter."""
@@ -1257,6 +1296,7 @@ class LambdaFromMultipleTOVFiles(BaseTransform):
                                         lambda_param=self._lambda_param,
                                         mass_lambda_file=fname,
                                         distance=self._distance,
+                                        redshift_mass=self.redshift_mass,
                                         file_columns=self._file_columns)
                 self._eos_cache[eos_index] = eos
             except KeyError:
@@ -1276,6 +1316,19 @@ class LambdaFromMultipleTOVFiles(BaseTransform):
             out = {self._lambda_param : numpy.nan}
             return self.format_output(maps, out)
 
+    @classmethod
+    def from_config(cls, cp, section, outputs):
+        # see if we're redshifting masses
+        if cp.has_option('-'.join([section, outputs]), 'do-not-redshift-mass'):
+            additional_opts = {'redshift_mass': False}
+            skip_opts = ['do-not-redshift-mass']
+        else:
+            additional_opts = None
+            skip_opts = None
+        return super(LambdaFromMultipleTOVFiles, cls).from_config(
+            cp, section, outputs, skip_opts=skip_opts,
+            additional_opts=additional_opts)
+
 
 class Log(BaseTransform):
     """Applies a log transform from an `inputvar` parameter to an `outputvar`
@@ -1852,7 +1905,7 @@ class ChiPToCartesianSpin(CartesianSpinToChiP):
 
 class Exponent(Log):
     """Applies an exponent transform to an `inputvar` parameter.
-    
+
     This is the inverse of the log transform.
 
     Parameters
@@ -2053,7 +2106,7 @@ common_cbc_inverse_transforms.extend([
     CartesianToSpherical(parameters.spin2x, parameters.spin2y,
                          parameters.spin2z, parameters.spin2_a,
                          parameters.spin2_azimuthal, parameters.spin2_polar)])
-    
+
 common_cbc_transforms = common_cbc_forward_transforms + \
                         common_cbc_inverse_transforms
 
