@@ -24,10 +24,13 @@
 """
 This modules provides classes for generating waveforms.
 """
+import os
+import logging
 
 from . import waveform
 from .waveform import (NoWaveformError, FailedWaveformError)
 from . import ringdown
+from . import supernovae
 from pycbc import filter
 from pycbc import transforms
 from pycbc.types import TimeSeries
@@ -35,15 +38,17 @@ from pycbc.waveform import parameters
 from pycbc.waveform.utils import apply_fd_time_shift, taper_timeseries, \
                                  ceilpow2
 from pycbc.detector import Detector
+from pycbc.pool import use_mpi
 import lal as _lal
 from pycbc import strain
-import logging
+
 
 #
 #   Generator for CBC waveforms
 #
 
 # utility functions/class
+failed_counter = 0
 
 class BaseGenerator(object):
     """A wrapper class to call a waveform generator with a set of frozen
@@ -61,6 +66,9 @@ class BaseGenerator(object):
         A tuple or list of strings giving the names and order of variable
         parameters that will be passed to the waveform generator when the
         generate function is called.
+    record_failures : boolean
+        Store output files containing the parameters of failed waveform
+        generation. Default is False.
     \**frozen_params :
         These keyword arguments are the ones that will be frozen in the
         waveform generator. For a list of possible parameters, see
@@ -85,7 +93,8 @@ class BaseGenerator(object):
         Generates a waveform using the variable arguments and the frozen
         arguments.
     """
-    def __init__(self, generator, variable_args=(), **frozen_params):
+    def __init__(self, generator, variable_args=(), record_failures=False,
+                 **frozen_params):
         self.generator = generator
         self.variable_args = tuple(variable_args)
         self.frozen_params = frozen_params
@@ -94,6 +103,15 @@ class BaseGenerator(object):
         self.current_params = frozen_params.copy()
         # keep a list of functions to call before waveform generation
         self._pregenerate_functions = []
+        # we'll cache the last generated waveform here
+        self._cache = {}
+
+        # If we are under mpi, then failed waveform will be stored by
+        # mpi rank to avoid file writing conflicts. We'll check for this
+        # upfront
+        self.record_failures = (record_failures or
+                                ('PYCBC_RECORD_FAILED_WAVEFORMS' in os.environ))
+        self.mpi_enabled, _, self.mpi_rank = use_mpi()
 
     @property
     def static_args(self):
@@ -113,7 +131,6 @@ class BaseGenerator(object):
         """
         self._pregenerate_functions.append(func)
 
-
     def _postgenerate(self, res):
         """Allows the waveform returned by the generator function to be
         manipulated before returning.
@@ -135,16 +152,43 @@ class BaseGenerator(object):
     def _generate_from_current(self):
         """Generates a waveform from the current parameters.
         """
+        values_hash = frozenset(self.current_params[p]
+                                for p in self.variable_args)
         try:
-            return self.generator(**self.current_params)
-        except RuntimeError as e:
-            # we'll get a RuntimeError if lalsimulation failed to generate
-            # the waveform for whatever reason
-            strparams = ' | '.join(['{}: {}'.format(p, str(val))
-                                   for p, val in self.current_params.items()])
-            raise FailedWaveformError("Failed to generate waveform with "
-                                      "parameters:\n{}\nError was: {}"
-                                      .format(strparams, e))
+            return self._cache[values_hash]
+        except KeyError:
+            try:
+                new_waveform = self.generator(**self.current_params)
+                self._cache.clear()
+                self._cache[values_hash] = new_waveform
+                return new_waveform
+            except RuntimeError as e:
+                if self.record_failures:
+                    import h5py
+                    from pycbc.io.hdf import dump_state
+
+                    global failed_counter
+
+                    if self.mpi_enabled:
+                        outname = 'failed/params_%s.hdf' % self.mpi_rank
+                    else:
+                        outname = 'failed/params.hdf'
+
+                    if not os.path.exists('failed'):
+                        os.makedirs('failed')
+
+                    with h5py.File(outname) as f:
+                        dump_state(self.current_params, f,
+                                   dsetname=str(failed_counter))
+                        failed_counter += 1
+
+                # we'll get a RuntimeError if lalsimulation failed to generate
+                # the waveform for whatever reason
+                strparams = ' | '.join(['{}: {}'.format(
+                    p, str(val)) for p, val in self.current_params.items()])
+                raise FailedWaveformError("Failed to generate waveform with "
+                                          "parameters:\n{}\nError was: {}"
+                                          .format(strparams, e))
 
 
 class BaseCBCGenerator(BaseGenerator):
@@ -399,6 +443,205 @@ class TDomainFreqTauRingdownGenerator(BaseGenerator):
         super(TDomainFreqTauRingdownGenerator, self).__init__(ringdown.get_td_from_freqtau,
             variable_args=variable_args, **frozen_params)
 
+
+class FDomainDetFrameTwoPolGenerator(object):
+    """Generates frequency-domain waveform in a specific frame.
+
+    Generates both polarizations of a waveform using the given radiation frame
+    generator class, and applies the time shift. Detector response functions
+    are not applied.
+
+    Parameters
+    ----------
+    rFrameGeneratorClass : class
+        The class to use for generating the waveform in the radiation frame,
+        e.g., FDomainCBCGenerator. This should be the class, not an
+        instance of the class (the class will be initialized with the
+        appropriate arguments internally).
+    detectors : {None, list of strings}
+        The names of the detectors to use. If provided, all location parameters
+        must be included in either the variable args or the frozen params. If
+        None, the generate function will just return the plus polarization
+        returned by the rFrameGeneratorClass shifted by any desired time shift.
+    epoch : {float, lal.LIGOTimeGPS
+        The epoch start time to set the waveform to. A time shift = tc - epoch is
+        applied to waveforms before returning.
+    variable_args : {(), list or tuple}
+        A list or tuple of strings giving the names and order of parameters
+        that will be passed to the generate function.
+    \**frozen_params
+        Keyword arguments setting the parameters that will not be changed from
+        call-to-call of the generate function.
+
+    Attributes
+    ----------
+    location_args : set(['tc', 'ra', 'dec'])
+        The set of location parameters. These are not passed to the rFrame
+        generator class; instead, they are used to apply the detector response
+        function and/or shift the waveform in time. The parameters are:
+
+          * tc: The GPS time of coalescence (should be geocentric time).
+          * ra: Right ascension.
+          * dec: declination
+
+        All of these must be provided in either the variable args or the
+        frozen params if detectors is not None. If detectors
+        is None, tc may optionally be provided.
+
+    Attributes
+    ----------
+    detectors : dict
+        The dictionary of detectors that antenna patterns are calculated for
+        on each call of generate. If no detectors were provided, will be
+        ``{'RF': None}``, where "RF" means "radiation frame".
+    detector_names : list
+        The list of detector names. If no detectors were provided, then this
+        will be ['RF'] for "radiation frame".
+    epoch : lal.LIGOTimeGPS
+        The GPS start time of the frequency series returned by the generate function.
+        A time shift is applied to the waveform equal to tc-epoch. Update by using
+        ``set_epoch``.
+    current_params : dict
+        A dictionary of name, value pairs of the arguments that were last
+        used by the generate function.
+    rframe_generator : instance of rFrameGeneratorClass
+        The instance of the radiation-frame generator that is used for waveform
+        generation. All parameters in current_params except for the
+        location params are passed to this class's generate function.
+    frozen_location_args : dict
+        Any location parameters that were included in the frozen_params.
+    variable_args : tuple
+        The list of names of arguments that are passed to the generate
+        function.
+
+    """
+    location_args = set(['tc', 'ra', 'dec'])
+
+    def __init__(self, rFrameGeneratorClass, epoch, detectors=None,
+                 variable_args=(), recalib=None, gates=None, **frozen_params):
+        # initialize frozen & current parameters:
+        self.current_params = frozen_params.copy()
+        self._static_args = frozen_params.copy()
+        # we'll separate out frozen location parameters from the frozen
+        # parameters that are sent to the rframe generator
+        self.frozen_location_args = {}
+        loc_params = set(frozen_params.keys()) & self.location_args
+        for param in loc_params:
+            self.frozen_location_args[param] = frozen_params.pop(param)
+        # set the order of the variable parameters
+        self.variable_args = tuple(variable_args)
+        # variables that are sent to the rFrame generator
+        rframe_variables = list(set(self.variable_args) - self.location_args)
+        # initialize the radiation frame generator
+        self.rframe_generator = rFrameGeneratorClass(
+            variable_args=rframe_variables, **frozen_params)
+        self.set_epoch(epoch)
+        # set calibration model
+        self.recalib = recalib
+        # if detectors are provided, convert to detector type; also ensure that
+        # location variables are specified
+        if detectors is not None:
+            self.detectors = {det: Detector(det) for det in detectors}
+            missing_args = [arg for arg in self.location_args if not
+                (arg in self.current_params or arg in self.variable_args)]
+            if any(missing_args):
+                raise ValueError("detectors provided, but missing location "
+                    "parameters %s. " %(', '.join(missing_args)) +
+                    "These must be either in the frozen params or the "
+                    "variable args.")
+        else:
+            self.detectors = {'RF': None}
+        self.detector_names = sorted(self.detectors.keys())
+        self.gates = gates
+
+    def set_epoch(self, epoch):
+        """Sets the epoch; epoch should be a float or a LIGOTimeGPS."""
+        self._epoch = float(epoch)
+
+    @property
+    def static_args(self):
+        """Returns a dictionary of the static arguments."""
+        return self._static_args
+
+    @property
+    def epoch(self):
+        return _lal.LIGOTimeGPS(self._epoch)
+
+    def generate(self, **kwargs):
+        """Generates a waveform polarizations and applies a time shift.
+
+        Returns
+        -------
+        dict :
+            Dictionary of ``detector names -> (hp, hc)``, where ``hp, hc`` are
+            the plus and cross polarization, respectively.
+        """
+        self.current_params.update(kwargs)
+        rfparams = {param: self.current_params[param]
+            for param in kwargs if param not in self.location_args}
+        hp, hc = self.rframe_generator.generate(**rfparams)
+        if isinstance(hp, TimeSeries):
+            df = self.current_params['delta_f']
+            hp = hp.to_frequencyseries(delta_f=df)
+            hc = hc.to_frequencyseries(delta_f=df)
+            # time-domain waveforms will not be shifted so that the peak amp
+            # happens at the end of the time series (as they are for f-domain),
+            # so we add an additional shift to account for it
+            tshift = 1./df - abs(hp._epoch)
+        else:
+            tshift = 0.
+        hp._epoch = hc._epoch = self._epoch
+        h = {}
+        if self.detector_names != ['RF']:
+            for detname, det in self.detectors.items():
+                # apply the time shift
+                tc = self.current_params['tc'] + \
+                    det.time_delay_from_earth_center(self.current_params['ra'],
+                         self.current_params['dec'], self.current_params['tc'])
+                dethp = apply_fd_time_shift(hp, tc+tshift, copy=True)
+                dethc = apply_fd_time_shift(hc, tc+tshift, copy=True)
+                if self.recalib:
+                    # recalibrate with given calibration model
+                    dethp = self.recalib[detname].map_to_adjust(
+                        dethp, **self.current_params)
+                    dethc = self.recalib[detname].map_to_adjust(
+                        dethc, **self.current_params)
+                h[detname] = (dethp, dethc)
+        else:
+            # no detector response, just use the + polarization
+            if 'tc' in self.current_params:
+                hp = apply_fd_time_shift(hp, self.current_params['tc']+tshift,
+                                         copy=False)
+                hc = apply_fd_time_shift(hc, self.current_params['tc']+tshift,
+                                         copy=False)
+            h['RF'] = (hp, hc)
+        if self.gates is not None:
+            # resize all to nearest power of 2
+            hps = {}
+            hcs = {}
+            for det in h:
+                hp = h[det]
+                hc = h[det]
+                hp.resize(ceilpow2(len(hp)-1) + 1)
+                hc.resize(ceilpow2(len(hc)-1) + 1)
+                hps[det] = hp
+                hcs[det] = hc
+            hps = strain.apply_gates_to_fd(hps, self.gates)
+            hcs = strain.apply_gates_to_fd(hps, self.gates)
+            h = {det: (hps[det], hcs[det]) for det in h}
+        return h
+
+
+class TDomainSupernovaeGenerator(BaseGenerator):
+    """Uses supernovae.py to create time domain core-collapse supernovae waveforms
+    using a set of Principal Components provided in a .hdf file.
+    """
+    def __init__(self, variable_args=(), **frozen_params):
+        super(TDomainSupernovaeGenerator,
+              self).__init__(supernovae.get_corecollapse_bounce,
+           variable_args=variable_args, **frozen_params)
+
+
 class FDomainDetFrameGenerator(object):
     """Generates frequency-domain waveform in a specific frame.
 
@@ -587,6 +830,7 @@ class FDomainDetFrameGenerator(object):
         return h
 
 
+
 def select_waveform_generator(approximant):
     """Returns the single-IFO generator for the approximant.
 
@@ -636,6 +880,11 @@ def select_waveform_generator(approximant):
         elif approximant == 'TdQNMfromFreqTau':
             return TDomainFreqTauRingdownGenerator
 
+    # check if supernovae waveform:
+    elif approximant in supernovae.supernovae_td_approximants:
+        if approximant == 'CoreCollapseBounce':
+            return TDomainSupernovaeGenerator
+
     # otherwise waveform approximant is not supported
     else:
         raise ValueError("%s is not a valid approximant." % approximant)
