@@ -68,29 +68,26 @@ def format_lmns(lmns):
     """ Checks if the format of the parameter lmns is correct, returning the
     appropriate format if not, and raise an error if nmodes=0.
     The required format for the ringdown approximants is a list of lmn modes
-    as strings, with n the number of overtones desired. For instance,
-    lmns = ['223','331'] are the modes 220, 221, 222, and 330.
+    as a single whitespace-separated string, with n the number
+    of overtones desired. Alternatively, a list object may be used,
+    containing individual strings as elements for the lmn modes.
+    For instance, lmns = '223 331' are the modes 220, 221, 222, and 330.
+    Giving lmns = ['223', '331'] is equivalent.
     The ConfigParser of a workflow might convert that to a single string
     (case 1 below) or a list with a single string (case 2), and this function
     will return the appropriate list of strings. If a different format is
     given, raise an error.
     """
 
-    # Case 1: the list is in a string "['221', '331']"
-    # In Python3 this might be "[b'221', b'331']"
+    # Case 1: the lmns are given as a string, e.g. '221 331'
     if isinstance(lmns, str):
-        # strip off brackets and convert to list
-        for char in ["[", "]", "'", " ", "b"]:
-            lmns = lmns.replace(char,'')
-        lmns = lmns.split(',')
-
-    # Case 2: a list with only one string with a list ["221', '331"]
-    # In Python3 this might be ["b221', b'331"]
-    elif (len(lmns) == 1 and isinstance(lmns[0], str)
-          and len(lmns[0]) > 3):
-        for char in ["[", "]", "'", " ", "b"]:
-            lmns[0] = lmns[0].replace(char,'')
-        lmns = lmns[0].split(',')
+        lmns = lmns.split(' ')
+    # Case 2: the lmns are given as strings in a list, e.g. ['221', '331']
+    elif isinstance(lmns, list):
+        pass
+    else:
+        raise ValueError('Format of parameter lmns not recognized. See '
+                         'approximant documentation for more info.')
 
     out = []
     # Cycle over the lmns to ensure that we get back a list of strings that
@@ -460,23 +457,41 @@ def fd_damped_sinusoid(f_0, tau, amp, phi, delta_f, f_lower, f_final, t_0=0.,
 #### Base multi-mode for all approximants
 ######################################################
 
-def multimode_base(input_params):
+def multimode_base(input_params, domain, freq_tau_approximant=False):
     """Return a superposition of damped sinusoids in either time or frequency
     domains with parameters set by input_params.
+
+    Parameters
+    ----------
+    domain : string
+        Choose domain of the waveform, either 'td' for time domain
+        or 'fd' for frequency domain.
+    freq_tau_approximant : {False, bool}, optional
+        Choose choose the waveform approximant to use. Either based on
+        mass/spin (set to False, default), or on frequencies/damping times
+        of the modes (set to True).
+
+    Returns
+    -------
+    hplus : TimeSeries
+        The plus phase of a ringdown with the lm modes specified and
+        n overtones in the chosen domain (time or frequency).
+    hcross : TimeSeries
+        The cross phase of a ringdown with the lm modes specified and
+        n overtones in the chosen domain (time or frequency).
     """
     input_params['lmns'] = format_lmns(input_params['lmns'])
     amps, phis = lm_amps_phases(**input_params)
-    if 'final_mass' in input_params.keys():
+    if freq_tau_approximant:
+        freqs, taus = lm_freqs_taus(**input_params)
+        norm = 1.
+    else:
         freqs, taus = get_lm_f0tau_allmodes(input_params['final_mass'],
                         input_params['final_spin'], input_params['lmns'])
         norm = Kerr_factor(input_params['final_mass'],
             input_params['distance']) if 'distance' in input_params.keys() \
             else 1.
-    else:
-        freqs, taus = lm_freqs_taus(**input_params)
-        norm = 1.
-
-    if 'delta_t' in input_params.keys():
+    if domain == 'td':
         outplus, outcross = td_output_vector(freqs, taus,
                             input_params['taper'], input_params['delta_t'],
                             input_params['t_final'])
@@ -502,17 +517,21 @@ def multimode_base(input_params):
             else:
                 outplus.data += hplus
                 outcross.data += hcross
-    elif 'delta_f' in input_params.keys():
+    elif domain == 'fd':
         outplus, outcross = fd_output_vector(freqs, taus,
                             input_params['delta_f'], input_params['f_final'])
         for lmn in freqs:
             hplus, hcross = fd_damped_sinusoid(freqs[lmn], taus[lmn],
                             amps[lmn], phis[lmn], outplus.delta_f,
                             input_params['f_lower'],
-                            outplus.sample_frequencies[-1], int(lmn[0]),
-                            int(lmn[1]), input_params['inclination'])
+                            input_params['f_final'],
+                            l=int(lmn[0]), m=int(lmn[1]),
+                            inclination=input_params['inclination'])
             outplus.data += hplus.data
             outcross.data += hcross.data
+    else:
+        raise ValueError('unrecognised domain argument {}; '
+                         'must be either fd or td'.format(domain))
 
     return norm * outplus, norm * outcross
 
@@ -580,7 +599,7 @@ def get_td_from_final_mass_spin(template=None, **kwargs):
 
     input_params = props(template, mass_spin_required_args, td_args, **kwargs)
 
-    return multimode_base(input_params)
+    return multimode_base(input_params, domain='td')
 
 def get_fd_from_final_mass_spin(template=None, **kwargs):
     """Return frequency domain ringdown with all the modes specified.
@@ -640,7 +659,7 @@ def get_fd_from_final_mass_spin(template=None, **kwargs):
 
     input_params = props(template, mass_spin_required_args, fd_args, **kwargs)
 
-    return multimode_base(input_params)
+    return multimode_base(input_params, domain='fd')
 
 def get_td_from_freqtau(template=None, **kwargs):
     """Return time domain ringdown with all the modes specified.
@@ -700,7 +719,7 @@ def get_td_from_freqtau(template=None, **kwargs):
 
     input_params = props(template, freqtau_required_args, td_args, **kwargs)
 
-    return multimode_base(input_params)
+    return multimode_base(input_params, domain='td', freq_tau_approximant=True)
 
 def get_fd_from_freqtau(template=None, **kwargs):
     """Return frequency domain ringdown with all the modes specified.
@@ -756,7 +775,7 @@ def get_fd_from_freqtau(template=None, **kwargs):
 
     input_params = props(template, freqtau_required_args, fd_args, **kwargs)
 
-    return multimode_base(input_params)
+    return multimode_base(input_params, domain='fd', freq_tau_approximant=True)
 
 # Approximant names ###########################################################
 ringdown_fd_approximants = {'FdQNMfromFinalMassSpin': get_fd_from_final_mass_spin,
